from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.latest_cohort_prediction import LatestCohortPrediction
from ...models.latest_cohort_prediction_bad_request import (
    LatestCohortPredictionBadRequest,
)
from ...models.latest_cohort_prediction_unauthorized import (
    LatestCohortPredictionUnauthorized,
)
from ...models.prediction_service_get_latest_cohort_simplified_response_500 import (
    PredictionService_GetLatestCohortSimplifiedResponse_500,
)
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: Client,
    cohort: str,
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/cohort/latest".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    params: Dict[str, Any] = {
        "cohort": cohort,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        LatestCohortPrediction,
        LatestCohortPredictionBadRequest,
        LatestCohortPredictionUnauthorized,
        str,
        PredictionService_GetLatestCohortSimplifiedResponse_500,
    ]
]:
    if response.status_code == 200:
        response_200 = LatestCohortPrediction.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = LatestCohortPredictionBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = LatestCohortPredictionUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = response.json()
        return response_404
    if response.status_code == 500:
        response_500 = (
            PredictionService_GetLatestCohortSimplifiedResponse_500.from_dict(
                response.json()
            )
        )

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        LatestCohortPrediction,
        LatestCohortPredictionBadRequest,
        LatestCohortPredictionUnauthorized,
        str,
        PredictionService_GetLatestCohortSimplifiedResponse_500,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cohort: str,
) -> Response[
    Union[
        LatestCohortPrediction,
        LatestCohortPredictionBadRequest,
        LatestCohortPredictionUnauthorized,
        str,
        PredictionService_GetLatestCohortSimplifiedResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cohort: str,
) -> Optional[
    Union[
        LatestCohortPrediction,
        LatestCohortPredictionBadRequest,
        LatestCohortPredictionUnauthorized,
        str,
        PredictionService_GetLatestCohortSimplifiedResponse_500,
    ]
]:
    """
    Fetch most recent prepayment prediction for a single cohort.

    Args:
        cohort: Cohort name to fetch aggregated prepayment prediction.
    """

    return sync_detailed(
        client=client,
        cohort=cohort,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cohort: str,
) -> Response[
    Union[
        LatestCohortPrediction,
        LatestCohortPredictionBadRequest,
        LatestCohortPredictionUnauthorized,
        str,
        PredictionService_GetLatestCohortSimplifiedResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cohort=cohort,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cohort: str,
) -> Optional[
    Union[
        LatestCohortPrediction,
        LatestCohortPredictionBadRequest,
        LatestCohortPredictionUnauthorized,
        str,
        PredictionService_GetLatestCohortSimplifiedResponse_500,
    ]
]:
    """ Fetch most recent prepayment prediction for a single cohort. """

    return (
        await asyncio_detailed(
            client=client,
            cohort=cohort,
        )
    ).parsed
