from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.latest_latest_pool_predictions import LatestPoolPredictions
from ...models.latest_pool_predictions_bad_request import (
    LatestPoolPredictionsBadRequest,
)
from ...models.latest_pool_predictions_unauthorized import (
    LatestPoolPredictionsUnauthorized,
)
from ...models.prediction_service_get_latest_pools_simplified_response_500 import (
    PredictionService_GetLatestPoolsSimplifiedResponse_500,
)
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: Client,
    cusips: List[str],
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/pools/latest".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_cusips = cusips

    params: Dict[str, Any] = {
        "CUSIPS": json_cusips,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        LatestPoolPredictions,
        LatestPoolPredictionsBadRequest,
        LatestPoolPredictionsUnauthorized,
        str,
        PredictionService_GetLatestPoolsSimplifiedResponse_500,
    ]
]:
    if response.status_code == 200:
        response_200 = LatestPoolPredictions.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = LatestPoolPredictionsBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = LatestPoolPredictionsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = response.json()
        return response_404
    if response.status_code == 500:
        response_500 = PredictionService_GetLatestPoolsSimplifiedResponse_500.from_dict(
            response.json()
        )

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        LatestPoolPredictions,
        LatestPoolPredictionsBadRequest,
        LatestPoolPredictionsUnauthorized,
        str,
        PredictionService_GetLatestPoolsSimplifiedResponse_500,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cusips: List[str],
) -> Response[
    Union[
        LatestPoolPredictions,
        LatestPoolPredictionsBadRequest,
        LatestPoolPredictionsUnauthorized,
        str,
        PredictionService_GetLatestPoolsSimplifiedResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cusips: List[str],
) -> Optional[
    Union[
        LatestPoolPredictions,
        LatestPoolPredictionsBadRequest,
        LatestPoolPredictionsUnauthorized,
        str,
        PredictionService_GetLatestPoolsSimplifiedResponse_500,
    ]
]:
    """
    Fetch most recent individual pool prepayment predictions.

    Args:
        cusips: Pool CUSIP or CUSIPS to fetch individual prepayment predictions.
    """

    return sync_detailed(
        client=client,
        cusips=cusips,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cusips: List[str],
) -> Response[
    Union[
        LatestPoolPredictions,
        LatestPoolPredictionsBadRequest,
        LatestPoolPredictionsUnauthorized,
        str,
        PredictionService_GetLatestPoolsSimplifiedResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cusips: List[str],
) -> Optional[
    Union[
        LatestPoolPredictions,
        LatestPoolPredictionsBadRequest,
        LatestPoolPredictionsUnauthorized,
        str,
        PredictionService_GetLatestPoolsSimplifiedResponse_500,
    ]
]:
    """ Fetch most recent individual pool prepayment predictions. """

    return (
        await asyncio_detailed(
            client=client,
            cusips=cusips,
        )
    ).parsed
