from typing import Dict

import httpx

from .client import Client
from .prediction import prediction
from .version import __version__


class InfimaClient(Client):
    def __init__(
        self,
        token: str = None,
        base_url: str = "https://api.infima.io",
        extra_headers: Dict[str, str] = None,
    ):
        super(InfimaClient, self).__init__(base_url=base_url)
        if token:
            self.headers["INFIMA-TOKEN"] = token
        if extra_headers:
            self.headers.update(extra_headers)

        self.prediction = prediction(self)

    def check(self) -> bool:
        """Check:
        - version
        - authenticated connection
        """
        try:
            with httpx.Client() as client:
                r = client.get(f"{self.base_url}/info")
                version = r.json()["version"]
                print(
                    f"You are not using the most recent version of the client: {__version__}. Available: {version}."
                )
                if version != __version__:
                    print(
                        f"You are not using the most recent version of the client: {__version__}. Available: {version}."
                    )
        except:
            pass

        with httpx.Client(headers=self.headers) as client:
            r = client.get(f"{self.base_url}/api/v1/welcome")
            return r.status_code == 200
        return False
