from typing import Dict, List, Optional, Union

import infima_client.api.prediction_service.prediction_service_get_latest_cohort_simplified as prediction_service_get_latest_cohort_simplified
import infima_client.api.prediction_service.prediction_service_get_latest_pools_simplified as prediction_service_get_latest_pools_simplified
import infima_client.api.prediction_service.prediction_service_get_latest_portfolio_simplified as prediction_service_get_latest_portfolio_simplified
from infima_client.models import *

from .client import Client
from .types import UNSET, Response, Unset


class prediction:
    def __init__(self, client: Client):
        self.client = client
        self._detailed = predictionDetailed(client)

    def get_latest_cohort_simplified(
        self, cohort: str
    ) -> Optional[
        Union[
            LatestCohortPrediction,
            LatestCohortPredictionBadRequest,
            LatestCohortPredictionUnauthorized,
            str,
            PredictionService_GetLatestCohortSimplifiedResponse_500,
        ]
    ]:

        """
        Fetch most recent prepayment prediction for a single cohort.

        Args:
            cohort: Cohort name to fetch aggregated prepayment prediction.
        """

        return prediction_service_get_latest_cohort_simplified.sync(
            client=self.client, cohort=cohort
        )

    def get_latest_pools_simplified(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            LatestPoolPredictions,
            LatestPoolPredictionsBadRequest,
            LatestPoolPredictionsUnauthorized,
            str,
            PredictionService_GetLatestPoolsSimplifiedResponse_500,
        ]
    ]:

        """
        Fetch most recent individual pool prepayment predictions.

        Args:
            cusips: Pool CUSIP or CUSIPS to fetch individual prepayment predictions.
        """

        return prediction_service_get_latest_pools_simplified.sync(
            client=self.client, cusips=cusips
        )

    def get_latest_portfolio_simplified(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            LatestPortfolioPrediction,
            LatestPortfolioPredictionBadRequest,
            LatestPortfolioPredictionUnauthorized,
            str,
            PredictionService_GetLatestPortfolioSimplifiedResponse_500,
        ]
    ]:

        """
        Weighted aggregate of the prepayment prediction of the portfolio components.

        Args:
            cusips: Pool CUSIPS to fetch aggregated prepayment prediction.
        """

        return prediction_service_get_latest_portfolio_simplified.sync(
            client=self.client, cusips=cusips
        )


class predictionDetailed:
    def __init__(self, client: Client):
        self.client = client

    def get_latest_cohort_simplified(
        self, cohort: str
    ) -> Response[
        Union[
            LatestCohortPrediction,
            LatestCohortPredictionBadRequest,
            LatestCohortPredictionUnauthorized,
            str,
            PredictionService_GetLatestCohortSimplifiedResponse_500,
        ]
    ]:

        return prediction_service_get_latest_cohort_simplified.sync_detailed(
            client=self.client, cohort=cohort
        )

    def get_latest_pools_simplified(
        self, cusips: List[str]
    ) -> Response[
        Union[
            LatestPoolPredictions,
            LatestPoolPredictionsBadRequest,
            LatestPoolPredictionsUnauthorized,
            str,
            PredictionService_GetLatestPoolsSimplifiedResponse_500,
        ]
    ]:

        return prediction_service_get_latest_pools_simplified.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_latest_portfolio_simplified(
        self, cusips: List[str]
    ) -> Response[
        Union[
            LatestPortfolioPrediction,
            LatestPortfolioPredictionBadRequest,
            LatestPortfolioPredictionUnauthorized,
            str,
            PredictionService_GetLatestPortfolioSimplifiedResponse_500,
        ]
    ]:

        return prediction_service_get_latest_portfolio_simplified.sync_detailed(
            client=self.client, cusips=cusips
        )
